#include "libr2c2/track.h"
#include "slopetool.h"

using namespace std;
using namespace Msp;
using namespace R2C2;

SlopeTool::SlopeTool(Designer &d, Input::Keyboard &k, Input::Mouse &m, const set<Object *> &objects):
	Tool(d, k, m),
	total_length(0)
{
	if(objects.empty())
	{
		set_status("Nothing selected");
		set_done(false);
		return;
	}

	for(set<Object *>::const_iterator i=objects.begin(); i!=objects.end(); ++i)
		if(!dynamic_cast<Track *>(*i) || (*i)->get_n_link_slots()!=2)
		{
			set_status("Must have linear tracks only");
			set_done(false);
			return;
		}

	TrackIter track;
	for(set<Object *>::const_iterator i=objects.begin(); (!track && i!=objects.end()); ++i)
		if(Track *t = dynamic_cast<Track *>(*i))
		{
			unsigned nls = t->get_n_link_slots();
			for(unsigned j=0; (!track && j<nls); ++j)
				if(!objects.count((*i)->get_link(j)))
					track = TrackIter(t, j);
		}

	total_length = 0;
	while(track && objects.count(track.track()))
	{
		tracks.push_back(track);
		total_length += track->get_type().get_path_length(0);
		track = track.next();
	}
}

void SlopeTool::even_slope(bool smooth)
{
	if(done)
		return;

	float start_z = tracks.front()->get_snap_node(tracks.front().entry()).position.z;
	float end_z = tracks.back()->get_snap_node(tracks.back().reverse().entry()).position.z;

	float length = total_length;
	if(smooth)
	{
		float dir = (end_z>start_z)?1:-1;
		float cur_slope = 0;
		while((end_z-start_z)*dir/length>cur_slope+0.025 && tracks.size()>2)
		{
			cur_slope += 0.025;
			Angle tilt = Geometry::atan(cur_slope);

			set_slope(tracks.front(), start_z, tilt);
			start_z += tracks.front()->get_type().get_path_length(0)*dir*cur_slope;
			length -= tracks.front()->get_type().get_path_length(0);
			tracks.pop_front();

			end_z -= tracks.back()->get_type().get_path_length(0)*dir*cur_slope;
			set_slope(tracks.back(), end_z, tilt);
			length -= tracks.back()->get_type().get_path_length(0);
			tracks.pop_back();
		}
	}

	float cur_z = start_z;
	Angle tilt = Geometry::atan((end_z-start_z)/length);
	for(list<TrackIter>::iterator i=tracks.begin(); i!=tracks.end(); ++i)
	{
		set_slope(*i, cur_z, tilt);
		cur_z += (*i)->get_type().get_path_length(0)*(end_z-start_z)/length;
	}

	set_done(true);
}

void SlopeTool::flatten()
{
	if(done)
		return;

	float z = 0;
	for(list<TrackIter>::iterator i=tracks.begin(); i!=tracks.end(); ++i)
	{
		unsigned nsn = (*i)->get_n_snap_nodes();
		for(unsigned j=0; j<nsn; ++j)
			z += (*i)->get_snap_node(j).position.z/nsn;
	}
	z /= static_cast<int>(tracks.size());

	for(list<TrackIter>::iterator i=tracks.begin(); i!=tracks.end(); ++i)
	{
		Vector p = (*i)->get_position();
		(*i)->set_position(Vector(p.x, p.y, z));
		(*i)->set_tilt(Angle::zero());
	}

	set_done(true);
}

void SlopeTool::set_slope(const TrackIter &track, float z, const Angle &tilt)
{
	const Vector &p = track->get_position();
	float dz = tan(tilt)*track->get_type().get_path_length(0);
	if(track.entry()==1)
	{
		track->set_position(Vector(p.x, p.y, z+dz));
		track->set_tilt(-tilt);
	}
	else
	{
		track->set_position(Vector(p.x, p.y, z));
		track->set_tilt(tilt);
	}
}
