#ifndef LIBR2C2_TRACKTYPE_H_
#define LIBR2C2_TRACKTYPE_H_

#include <msp/datafile/objectloader.h>
#include "geometry.h"
#include "objecttype.h"
#include "trackpart.h"

namespace R2C2 {

class TrackAppearance;

class TrackType: public ObjectType
{
public:
	struct Endpoint
	{
		Vector pos;
		Angle dir;  // Direction outwards from the endpoint
		unsigned paths;

		Endpoint(float, float, const Angle &, unsigned);

		bool has_path(unsigned p) const { return paths&(1<<p); }
		bool has_common_paths(const Endpoint &e) const { return paths&e.paths; }
	};

	class Loader: public Msp::DataFile::DerivedObjectLoader<TrackType, ObjectType::Loader>
	{
	public:
		typedef Msp::DataFile::Collection Collection;

	private:
		Collection &coll;
		bool state_bits_set;

	public:
		Loader(TrackType &, Collection &);

		virtual Collection &get_collection() const { return coll; }
	private:
		virtual void finish();
		void appearance(const std::string &);
		void part();
		void state_bits(unsigned);
	};

private:
	const TrackAppearance *appearance;
	std::vector<TrackPart> parts;
	std::vector<Endpoint> endpoints;
	unsigned state_bits;
	unsigned autofit_preference;
	std::string object;

public:
	TrackType();

	const TrackAppearance &get_appearance() const;
	float get_gauge() const;
	float get_total_length() const;
	float get_path_length(int) const;
	unsigned get_paths() const;
	unsigned get_n_paths() const;
	unsigned coerce_path(unsigned, unsigned) const;
	unsigned get_state_bits() const { return state_bits; }
	bool is_turnout() const;
	bool is_dead_end() const;
	unsigned get_autofit_preference() const { return autofit_preference; }
	const std::string &get_object() const { return object; }
	const std::vector<TrackPart> &get_parts() const { return parts; }
	const std::vector<Endpoint> &get_endpoints() const { return endpoints; }
	const Endpoint &get_endpoint(unsigned) const;
	OrientedPoint get_point(unsigned, unsigned, float) const;
	OrientedPoint get_nearest_point(const Vector &) const;

private:
	void collect_endpoints();
};

} // namespace R2C2

#endif
